
#ifndef KERNEL_BROCADE_CPLD_H
#define KERNEL_BROCADE_CPLD_H 1


/*
 * It is expected that the caller will clear/restore Linux INT
 * 	(SYS_INT_SAVE() & SYS_INT_RESTORE) before calling these
 *	two CP macros for Read-Modify-Write register operations
 *	if they will be modified inside any critical section.
 */
#define SYS_CP_INT_DISABLE(bMap, reg)	{\
	u_char value = *reg;		 \
	value |= (bMap);		 \
	*reg = value; 			}

#define SYS_CP_INT_ENABLE(bMap, reg)	{\
	u_char value = *reg;		 \
	value &= ~(bMap);		 \
	*reg = value;			}

/* general Read-Modify-Write register operation macros */
#define CPLD_SET_MASK(bMap, reg)	SYS_CP_INT_DISABLE(bMap, reg)
#define CPLD_CLEAR_MASK(bMap, reg)	SYS_CP_INT_ENABLE(bMap, reg)

#define ODIN_SYS_RESET_SHUTDOWN_REG           0x75
#define ODIN_SYSTEM_SHUTDOWN                  0x00
#define ODIN_CPLD_BASE_ADDR                   0xF8000000ULL

/*
 * void cpld_delay()
 *
 * Description
 *   Function to put process to sleep for
 *   a specified number of milliseconds
 *
 * Input(s):
 *  msec - Number of milliseconds to sleep
 *
 * Output(s):
 *  None
 *
 * Returns:
 *  N/A
 *
 */
extern inline void
cpld_delay(u_short msec)
{
  unsigned long timeout = ((HZ * msec) / 1000L);

  current->state = TASK_INTERRUPTIBLE;
  schedule_timeout(timeout);

}

extern void *cpld_get_base_addr(void);
extern void cpld_power_off(void);
extern void cpld_power_off_odin(void);


#endif  /* KERNEL_BROCADE_CPLD_H */
