/*
 * arch/ppc/platforms/85xx/silkworm8548.h
 *
 * Based on:
 * arch/ppc/platforms/85xx/mpc85xx_cds_common.h
 *
 * MPC85xx CDS board definitions
 *
 * Maintainer: Kumar Gala <kumar.gala@freescale.com>
 *
 * Copyright 2004 Freescale Semiconductor, Inc
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 *
 * Brocade Silkworm platform specific definitions
 *
 */

#ifndef __MACH_SILKWORM8548_H__
#define __MACH_SILKWORM8548_H__

#include <linux/config.h>
#include <linux/types.h>
#include <linux/serial.h>
#include <asm/ppcboot.h>
#include <linux/initrd.h>
#include <syslib/ppc85xx_setup.h>

#define BOARD_CCSRBAR           ((uint)0xf7000000)
#define CCSRBAR_SIZE            ((uint)1024*1024)

/* CADMUS info */
#define CADMUS_BASE (0xf8004000)
#define CADMUS_SIZE (256)
#define CM_VER	(0)
#define CM_CSR	(1)
#define CM_RST	(2)

/* CDS NVRAM/RTC */
#define CDS_RTC_ADDR	(0xf8000000)
#define CDS_RTC_SIZE	(8 * 1024)

/* PCI config */
#define PCI1_CFG_ADDR_OFFSET	(0x8000)
#define PCI1_CFG_DATA_OFFSET	(0x8004)







#define PCI2_CFG_ADDR_OFFSET	(0x9000)
#define PCI2_CFG_DATA_OFFSET	(0x9004)

/* PCI interrupt controller */
#define PIRQ0A                   MPC85xx_IRQ_EXT0
#define PIRQ0B                   MPC85xx_IRQ_EXT1
#define PIRQ0C                   MPC85xx_IRQ_EXT2
#define PIRQ0D                   MPC85xx_IRQ_EXT3
#define PIRQ1A                   MPC85xx_IRQ_EXT11

/* PCI 1 memory map */

#if defined(CONFIG_SILKWORM_COMMON)

extern unsigned long mpc85xx_pci1_lower_mem;
extern unsigned long mpc85xx_pci1_upper_mem;
extern unsigned long mpc85xx_pci1_mem_offset;

extern int mpc85xx_pci1hose_first_busno;
extern int mpc85xx_pci1hose_last_busno;
extern int mpc85xx_pci1hose_bus_offset;

#define MPC85XX_PCI1_LOWER_IO        0x00000000
#define MPC85XX_PCI1_UPPER_IO        0x00ffffff

#define MPC85XX_PCI1_LOWER_MEM       (mpc85xx_pci1_lower_mem)
#define MPC85XX_PCI1_UPPER_MEM       (mpc85xx_pci1_upper_mem)


extern unsigned long mpc85xx_pex_lower_mem;
extern unsigned long mpc85xx_pex_upper_mem;
extern unsigned long mpc85xx_pex_mem_offset;

#define MPC85XX_PEX_LOWER_MEM (mpc85xx_pex_lower_mem)
#define MPC85XX_PEX_UPPER_MEM (mpc85xx_pex_upper_mem)
#define MPC85XX_PEX_MEM_OFFSET (mpc85xx_pex_mem_offset)

#define MPC85XX_PEX_LOWER_IO 0x00000000
#define MPC85XX_PEX_UPPER_IO 0x00ffffff

#define MPC85XX_PEX_IO_BASE (0xe3000000)
#define MPC85XX_PEX_IO_SIZE (0x1000000)

#define PEX_CFG_ADDR_OFFSET (0xa000)
#define PEX_CFG_DATA_OFFSET (0xa004) 


#define MPC85XX_PCI1_IO_BASE         0xe2000000
#define MPC85XX_PCI1_MEM_OFFSET      (mpc85xx_pci1_mem_offset)





#define MPC85XX_PCI1_IO_SIZE         0x01000000

#define MPC85XX_PCI1_FIRST_BUSNO (mpc85xx_pci1hose_first_busno)
#define MPC85XX_PCI1_LAST_BUSNO (mpc85xx_pci1hose_last_busno)
#define MPC85XX_PCI1_BUS_OFFSET (mpc85xx_pci1hose_bus_offset)

extern unsigned long mpc85xx_pex_lower_mem;
extern unsigned long mpc85xx_pex_upper_mem;
extern unsigned long mpc85xx_pex_mem_offset;

extern int mpc85xx_pex1hose_first_busno;
extern int mpc85xx_pex1hose_last_busno;
extern int mpc85xx_pex1hose_bus_offset;

#define MPC85XX_PEX_LOWER_MEM (mpc85xx_pex_lower_mem)
#define MPC85XX_PEX_UPPER_MEM (mpc85xx_pex_upper_mem)
#define MPC85XX_PEX_MEM_OFFSET (mpc85xx_pex_mem_offset)

#define MPC85XX_PEX_LOWER_IO 0x00000000
#define MPC85XX_PEX_UPPER_IO 0x00ffffff

#define MPC85XX_PEX_IO_BASE (0xe3000000)
#define MPC85XX_PEX_IO_SIZE (0x1000000)

#define PEX_CFG_ADDR_OFFSET (0xa000)
#define PEX_CFG_DATA_OFFSET (0xa004) 

#define MPC85XX_PEX1_FIRST_BUSNO (mpc85xx_pex1hose_first_busno)
#define MPC85XX_PEX1_LAST_BUSNO (mpc85xx_pex1hose_last_busno)
#define MPC85XX_PEX1_BUS_OFFSET (mpc85xx_pex1hose_bus_offset)

/* Defaults are for Zentron co-CPU */
#define MPC85XX_PEX_COCPU_LOWER_MEM		0x80000000
#define MPC85XX_PEX_COCPU_UPPER_MEM		0xBFFFFFFF
#define MPC85XX_PEX_COCPU_MEM_OFFSET	0x00000000

#else

#define MPC85XX_PCI1_LOWER_IO        0x00000000
#define MPC85XX_PCI1_UPPER_IO        0x00ffffff

#define MPC85XX_PCI1_LOWER_MEM       0x80000000
#define MPC85XX_PCI1_UPPER_MEM       0x9fffffff

#define MPC85XX_PCI1_IO_BASE         0xe2000000
#define MPC85XX_PCI1_MEM_OFFSET      0x00000000

#define MPC85XX_PCI1_IO_SIZE         0x01000000

#define MPC85XX_PCI1_FIRST_BUSNO 0x0
#define MPC85XX_PCI1_LAST_BUSNO 0xff
#define MPC85XX_PCI1_BUS_OFFSET 0x00

#define MPC85XX_PEX1_FIRST_BUSNO 0x0
#define MPC85XX_PEX1_LAST_BUSNO 0xff
#define MPC85XX_PEX1_BUS_OFFSET 0x00

#endif

/* SILKWORM does not use the PCI2 */

/* PCI 2 memory map */
/* Note: the standard PPC fixups will cause IO space to get bumped by
 * hose->io_base_virt - isa_io_base => MPC85XX_PCI1_IO_SIZE */
#define MPC85XX_PCI2_LOWER_IO        0x00000000
#define MPC85XX_PCI2_UPPER_IO        0x00ffffff

#define MPC85XX_PCI2_LOWER_MEM       0xa0000000
#define MPC85XX_PCI2_UPPER_MEM       0xbfffffff

#define MPC85XX_PCI2_IO_BASE         0xe3000000
#define MPC85XX_PCI2_MEM_OFFSET      0x00000000

#define MPC85XX_PCI2_IO_SIZE         0x01000000

#define NR_8259_INTS		     16
#define CPM_IRQ_OFFSET		     NR_8259_INTS

/* The offset of USB intr register in CPLD */
#define SILKWORM_CPLD_USB_INTR_MASK_REG	0x6F
#define ELARA_CPLD_USB_INTR_MASK_REG	0x46
#define ODIN_CPLD_USB_INTR_MASK_REG    0x4B

/* Switch independent USB intr control routine */
extern int silkworm_get_usb_irqno(int);
extern int silkworm_mask_usb_intr(void);
extern int silkworm_unmask_usb_intr(void);

#endif /* __MACH_SILKWORM8548_H__ */
