/*
 * arch/ppc/syslib/ppc85xx_setup.c
 *
 * MPC85XX common board code
 *
 * Maintainer: Kumar Gala <kumar.gala@freescale.com>
 *
 * Copyright 2004 Freescale Semiconductor Inc.
 *
 * This program is free software; you can redistribute  it and/or modify it
 * under  the terms of  the GNU General  Public License as published by the
 * Free Software Foundation;  either version 2 of the  License, or (at your
 * option) any later version.
 */

#include <linux/config.h>
#include <linux/types.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/serial.h>
#include <linux/tty.h>	/* for linux/serial_core.h */
#include <linux/serial_core.h>
#include <linux/serial_8250.h>

#include <asm/time.h>
#include <asm/mpc85xx.h>
#include <asm/immap_85xx.h>
#include <asm/mmu.h>
#include <asm/ppc_sys.h>
#include <asm/kgdb.h>

#include <syslib/ppc85xx_setup.h>
#include <brocade/platform.h>
#include <brocade/silkworm.h>
#include <brocade/silkworm_features.h>
#include <platforms/4xx/silkworm_cpld.h>
#include <brocade/cpld.h>
#include <brocade/silkworm_hwsem.h>
#include <brocade/sin.h>

extern sw_bd silkworm;
extern void abort();


#define MPC8548_ERRATA(cpu_ver)     if(1)
#define PCI_BDF(b,d,f)  ((b) << 16 | (d) << 11 | (f) << 8)


#define PCI_CFG_OUT out_be32

/* ERRATA PCI-Ex 14 PEX Controller timeout */
#define PEX_FIX     out_be32(hose->cfg_addr+0x4, 0x0400ffff)
extern int silkworm_pci_standalone_scan;

static int
indirect_read_config_pex(struct pci_bus *bus, unsigned int devfn, int offset,
             int len, u32 *val)
{
    struct pci_controller *hose = bus->sysdata;
    volatile void __iomem *cfg_data;
    u32 temp;


    /* MPC8548E Errata PCI-Ex 28.
     * Possible artefact of CDCpp50937 needs further investigation */
/*
    MPC8548_ERRATA(1.1)
        if ((bus->number < 2) && (PCI_SLOT(devfn) != 0x0))
            return PCIBIOS_DEVICE_NOT_FOUND;
*/

    MPC8548_ERRATA(1.1) PEX_FIX;
    if (bus->number == hose->bus_offset)
        PCI_CFG_OUT(hose->cfg_addr,
            (0x80000000 | ((offset & 0xf00) << 16) |
             ((bus->number - hose->bus_offset) << 16)
             | (devfn << 8) | ((offset & 0xfc) )));
    else
        PCI_CFG_OUT(hose->cfg_addr,
            (0x80000000 | ((offset & 0xf00) << 16) |
             ((bus->number) << 16)
             | (devfn << 8) | ((offset & 0xfc) )));

    /*
     * Note: the caller has already checked that offset is
     * suitably aligned and that len is 1, 2 or 4.
     */
    /* ERRATA PCI-Ex 12 - Configuration Address/Data Alignment */
    cfg_data = hose->cfg_data;
    MPC8548_ERRATA(1.1) PEX_FIX;
    temp = in_le32(cfg_data);
    switch (len) {
    case 1:
        *val = (temp >> (((offset & 3))*8)) & 0xff;
        break;
    case 2:
        *val = (temp >> (((offset & 3))*8)) & 0xffff;
        break;
    default:
        *val = temp;
        break;
    }

    return PCIBIOS_SUCCESSFUL;
}

static int
indirect_write_config_pex(struct pci_bus *bus, unsigned int devfn, int offset,
              int len, u32 val)
{
    struct pci_controller *hose = bus->sysdata;
    volatile void __iomem *cfg_data;
    u32 temp;


    /* MPC8548E Errata PCI-Ex 28.
     * Possible artefact of CDCpp50937 needs further investigation */
    MPC8548_ERRATA(1.1)
        if ((bus->number <2) && (PCI_SLOT(devfn) != 0x0))
            return PCIBIOS_DEVICE_NOT_FOUND;

    MPC8548_ERRATA(1.1) PEX_FIX;
    if (bus->number == hose->bus_offset)
        PCI_CFG_OUT(hose->cfg_addr,
            (0x80000000 | ((offset & 0xf00) << 16) |
            ((bus->number - hose->bus_offset) << 16)
            | (devfn << 8) | ((offset & 0xfc) )));
    else
        PCI_CFG_OUT(hose->cfg_addr,
            (0x80000000 | ((offset & 0xf00) << 16) |
            ((bus->number) << 16)
            | (devfn << 8) | ((offset & 0xfc) )));

    /*
     * Note: the caller has already checked that offset is
     * suitably aligned and that len is 1, 2 or 4.
     */
    /* ERRATA PCI-Ex 12 - Configuration Address/Data Alignment */
    cfg_data = hose->cfg_data;
    switch (len) {
    case 1:
        MPC8548_ERRATA(1.1) PEX_FIX;
            temp = in_le32(cfg_data);
            temp = (temp & ~(0xff << ((offset & 3) * 8))) |
                (val << ((offset & 3) * 8));
        MPC8548_ERRATA(1.1) PEX_FIX;
            out_le32(cfg_data, temp);
        break;
    case 2:
        MPC8548_ERRATA(1.1) PEX_FIX;
            temp = in_le32(cfg_data);
            temp = (temp & ~(0xffff << ((offset & 3) * 8)));
            temp |= (val << ((offset & 3) * 8)) ;
        MPC8548_ERRATA(1.1) PEX_FIX;
            out_le32(cfg_data, temp);
        break;
    default:
        MPC8548_ERRATA(1.1) PEX_FIX;
        out_le32(cfg_data, val);
        break;
    }
    MPC8548_ERRATA(1.1) PEX_FIX;

    return PCIBIOS_SUCCESSFUL;
}


int mpc85xx_pci1_last_busno = 0;

static struct pci_ops indirect_pex_ops = {
    indirect_read_config_pex,
    indirect_write_config_pex
};



void __init
setup_indirect_pex_nomap(struct pci_controller* hose, void __iomem * cfg_addr,
    void __iomem * cfg_data)
{
    hose->cfg_addr = cfg_addr;
    hose->cfg_data = cfg_data;
    hose->ops = &indirect_pex_ops;
}


void __init
setup_indirect_pex(struct pci_controller* hose, u32 cfg_addr, u32 cfg_data)
{
    unsigned long base = cfg_addr & PAGE_MASK;
    void __iomem *mbase, *addr, *data;
    mbase = ioremap(base, PAGE_SIZE);
    addr = mbase + (cfg_addr & ~PAGE_MASK);
    if ((cfg_data & PAGE_MASK) != base)
        mbase = ioremap(cfg_data & PAGE_MASK, PAGE_SIZE);
    data = mbase + (cfg_data & ~PAGE_MASK);
    setup_indirect_pex_nomap(hose, addr, data);
}

#define MPC85xx_PEX_OFFSET (0xA000)
#define MPC85xx_PEX_SIZE (0x1000)

static void __init
mpc85xx_setup_pex(struct pci_controller *hose)
{
	volatile struct ccsr_pci *pci;
	unsigned short temps;
	bd_t *binfo = (bd_t *) __res;
	volatile u32 busses = 0;

	if (silkworm.board == SWBD68)
		return;

	pci = ioremap(binfo->bi_immr_base + MPC85xx_PEX_OFFSET,
		    MPC85xx_PEX_SIZE);

	early_read_config_word(hose, hose->bus_offset, 0, PCI_COMMAND, &temps);
	temps |= PCI_COMMAND_SERR | PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY;
	early_write_config_word(hose, hose->bus_offset, 0, PCI_COMMAND, temps);

 	if ((silkworm.board != SWBD76) && (silkworm.board != SWBD133)) {
	/* Ensure the busses setting matches what the hose supports */
	early_read_config_dword(hose, hose->bus_offset, 0, PCI_PRIMARY_BUS, &busses);
	busses = ((busses & 0xFF000000) |
			  (hose->last_busno << 16) | (hose->first_busno << 8));
	early_write_config_dword(hose, hose->bus_offset, 0, PCI_PRIMARY_BUS, busses);
	}

#if 1 
    /* @kevin : use the values setup by u-boot */
    if (!silkworm_pci_standalone_scan) {
        /* Brocade, Silkworm: Honour the values setup by u-boot */
        return;
    }
#endif

	/* Disable all windows (except powar0 since its ignored) */
	pci->powar1 = 0;
	pci->powar2 = 0;
	pci->powar3 = 0;
	pci->powar4 = 0;
	pci->piwar1 = 0;
	pci->piwar2 = 0;
	pci->piwar3 = 0;

	/* Setup Phys:PCI 1:1 outbound mem window @ MPC85XX_PEX_LOWER_MEM */
	pci->potar1 = (MPC85XX_PEX_LOWER_MEM >> 12) & 0x000fffff;
	pci->potear1 = 0x00000000;
	pci->powbar1 = (MPC85XX_PEX_LOWER_MEM >> 12) & 0x000fffff;
	/* Enable, Mem R/W */
	pci->powar1 = 0x80044000 |
	   (__ilog2(MPC85XX_PEX_UPPER_MEM - MPC85XX_PEX_LOWER_MEM + 1) - 1);

	/* Setup outbound IO windows @ MPC85XX_PEX_IO_BASE */
	pci->potar2 = (MPC85XX_PEX_LOWER_IO >> 12) & 0x000fffff;;
	pci->potear2 = 0x00000000;
	pci->powbar2 = (MPC85XX_PEX_IO_BASE >> 12) & 0x000fffff;
	/* Enable, IO R/W */
	pci->powar2 = 0x80088000 | (__ilog2(MPC85XX_PEX_IO_SIZE) - 1);

	/* Setup 2G inbound Memory Window @ 0 */
	pci->pitar1 = 0x00000000;

	pci->piwbar1 = 0x00000000;
	pci->piwar1 = 0xa0f5501e;	/* Enable, Prefetch, Local
					   Mem, Snoop R/W, 2G */
}

#define cfg_read(val, addr, type, op)   *val = op((type)(addr))
#define cfg_write(val, addr, type, op)  op((type *)(addr), (val))

#define sync()  __asm__ __volatile__ ("sync" : : : "memory");

#define PCI_BUS(d)  (((d) >> 16) & 0xff)
#define PCI_DEV(d)  (((d) >> 11) & 0x1f)
typedef int pci_dev_t;


int
pex_indirect_read_config_word(struct pci_controller *hose,
                              pci_dev_t dev, int offset, u16 *val)
{
    int busn;
    busn = PCI_BUS(dev);
    //printk(KERN_CRIT"Reading to bus: %d\n", busn);
    dev =  PCI_BDF(busn,PCI_DEV(dev), PCI_FUNC(dev));
    *(hose->cfg_addr) = dev | (offset & 0xfc) | 0x80000000;
    sync();
    cfg_read(val, hose->cfg_data + (offset & 2), u16 *, in_le16);
    return 0;

}



/* Return the amount of memory */
unsigned long __init
mpc85xx_find_end_of_memory(void)
{
        bd_t *binfo;

        binfo = (bd_t *) __res;

        return binfo->bi_memsize;
}

/* The decrementer counts at the system (internal) clock freq divided by 8 */
void __init
mpc85xx_calibrate_decr(void)
{
        bd_t *binfo = (bd_t *) __res;
        unsigned int freq, divisor;

        /* get the core frequency */
        freq = binfo->bi_busfreq;

        /* The timebase is updated every 8 bus clocks, HID0[SEL_TBCLK] = 0 */
        divisor = 8;
        tb_ticks_per_jiffy = freq / divisor / HZ;
        tb_to_us = mulhwu_scale_factor(freq / divisor, 1000000);

	/* Set the time base to zero */
	mtspr(SPRN_TBWL, 0);
	mtspr(SPRN_TBWU, 0);

	/* Clear any pending timer interrupts */
	mtspr(SPRN_TSR, TSR_ENW | TSR_WIS | TSR_DIS | TSR_FIS);

	/* Enable decrementer interrupt */
	mtspr(SPRN_TCR, TCR_DIE);
}

#ifdef CONFIG_SERIAL_8250
void __init
mpc85xx_early_serial_map(void)
{
#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	struct uart_port serial_req;
#endif
	struct plat_serial8250_port *pdata;
	bd_t *binfo = (bd_t *) __res;
	pdata = (struct plat_serial8250_port *) ppc_sys_get_pdata(MPC85xx_DUART);

#if 0 /* The swap is not done properly disabling it until fixed */
#if defined(CONFIG_SILKWORM8548)
	/*
	 * This looks little hacky but it saves changes at many other places
	 * Silkworm platform only uses UART1 and is connected to debug port
	 * Swap the base addresses so that ttyS0 is hooked to UART1.
	 */
	{
		unsigned long swp;
		swp = pdata[0].mapbase;
		pdata[0].mapbase = pdata[1].mapbase;
		pdata[1].mapbase = swp;
	}

#endif
#endif

	/* Setup serial port access */
	pdata[0].uartclk = binfo->bi_busfreq;
	pdata[0].mapbase += binfo->bi_immr_base;
	pdata[0].membase = ioremap(pdata[0].mapbase, MPC85xx_UART0_SIZE);

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	memset(&serial_req, 0, sizeof (serial_req));
	serial_req.iotype = SERIAL_IO_MEM;
	serial_req.mapbase = pdata[0].mapbase;
	serial_req.membase = pdata[0].membase;
	serial_req.regshift = 0;

	gen550_init(0, &serial_req);
#endif

	pdata[1].uartclk = binfo->bi_busfreq;
	pdata[1].mapbase += binfo->bi_immr_base;
	pdata[1].membase = ioremap(pdata[1].mapbase, MPC85xx_UART0_SIZE);

#if defined(CONFIG_SERIAL_TEXT_DEBUG) || defined(CONFIG_KGDB)
	/* Assume gen550_init() doesn't modify serial_req */
	serial_req.mapbase = pdata[1].mapbase;
	serial_req.membase = pdata[1].membase;

	gen550_init(1, &serial_req);
#endif
}
#endif

/* Reset Control Register */
#define CCSRBAR_RSTCR_OFFSET		0x000e00b0
#define CCSRBAR_RSTCR_VAL			0x2			/* set HRESET_REQ field */
void
mpc85xx_restart(char *cmd)
{
	bd_t *binfo = (bd_t *) __res;
	volatile uint *rstcr_reg;

	/*
	 * Workaround from Freescale: we must set the HRESET_REQ field in
	 * the Reset Control Reg prior to initiating an HRESET via DBCR0
	 * otherwise we get hung up in u-boot.
	 */
#if 0
	/* Map in the CCSRBAR Reset Control Reg in the dev specific utils section */
	 rstcr_reg = (uint *) ioremap(binfo->bi_immr_base + CCSRBAR_RSTCR_OFFSET,
		 							sizeof(uint));
	 *rstcr_reg = CCSRBAR_RSTCR_VAL;
#endif

	local_irq_disable();
	abort();
}

void abort()
{
        bd_t *binfo = (bd_t *) __res;
        volatile uint *rstcr_reg;

        /*
	 * 7/30/07 
	 * Correction to Eratta.
	 * Originally we were doing HRESET via HRESET_REQ and followed by
	 * HRESET via DBCR0
	 * According to Krishna we should never attempt HRESET via DBCR0
	 * Therefore we perform HRESET via HRESET_REQ of CPLD only
	 * otherwise we get hung up in u-boot
         */

        /* Map in the CCSRBAR Reset Control Reg in the dev specific utils section */
         rstcr_reg = (uint *) ioremap(binfo->bi_immr_base + CCSRBAR_RSTCR_OFFSET,
                                                                        sizeof(uint));
         *rstcr_reg = CCSRBAR_RSTCR_VAL;
	for (;;) {
	  //it is kicking the power watch dog
	  mtspr(SPRN_TSR, (TSR_ENW | TSR_WIS));
	} /* loop in case reset takes some delay */
}

void
mpc85xx_power_off(void)
{
	local_irq_disable();
	for(;;){
	  //it is kicking the power watch dog
	  mtspr(SPRN_TSR, (TSR_ENW | TSR_WIS));
	}
}

void
mpc85xx_halt(void)
{
	local_irq_disable();
	for(;;){
	  //it is kicking the power watch dog
	  mtspr(SPRN_TSR, (TSR_ENW | TSR_WIS));
	}
}

#ifdef CONFIG_PCI

#if defined(CONFIG_MPC8555_CDS) || defined(CONFIG_MPC8548_CDS)
extern void mpc85xx_cds_enable_via(struct pci_controller *hose);
extern void mpc85xx_cds_fixup_via(struct pci_controller *hose);
#endif

static void __init
mpc85xx_setup_pci1(struct pci_controller *hose)
{
	volatile struct ccsr_pci *pci;
	volatile struct ccsr_guts *guts;
	unsigned short temps;
	bd_t *binfo = (bd_t *) __res;

	pci = ioremap(binfo->bi_immr_base + MPC85xx_PCI1_OFFSET,
		    MPC85xx_PCI1_SIZE);

	guts = ioremap(binfo->bi_immr_base + MPC85xx_GUTS_OFFSET,
		    MPC85xx_GUTS_SIZE);

	early_read_config_word(hose, 0, 0, PCI_COMMAND, &temps);
	temps |= PCI_COMMAND_SERR | PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY;
	early_write_config_word(hose, 0, 0, PCI_COMMAND, temps);

#define PORDEVSR_PCI	(0x00800000)	/* PCI Mode */
	if (guts->pordevsr & PORDEVSR_PCI) {
 		early_write_config_byte(hose, 0, 0, PCI_LATENCY_TIMER, 0x80);
 	} else {
		/* PCI-X init */
		temps = PCI_X_CMD_MAX_SPLIT | PCI_X_CMD_MAX_READ
			| PCI_X_CMD_ERO | PCI_X_CMD_DPERR_E;
		early_write_config_word(hose, 0, 0, PCIX_COMMAND, temps);
	}


	if (!silkworm_pci_standalone_scan) {
		/* Brocade, Silkworm: Honour the values setup by u-boot */
		return;
	}

	printk("SILKWORM: Reprograming the POW on this platform\n");
	/* Disable all windows (except powar0 since its ignored) */
	pci->powar1 = 0;
	pci->powar2 = 0;
	pci->powar3 = 0;
	pci->powar4 = 0;
	pci->piwar1 = 0;
	pci->piwar2 = 0;
	pci->piwar3 = 0;

	/* Setup Phys:PCI 1:1 outbound mem window @ MPC85XX_PCI1_LOWER_MEM */
	pci->potar1 = (MPC85XX_PCI1_LOWER_MEM >> 12) & 0x000fffff;
	pci->potear1 = 0x00000000;
	pci->powbar1 = (MPC85XX_PCI1_LOWER_MEM >> 12) & 0x000fffff;
	/* Enable, Mem R/W */
	pci->powar1 = 0x80044000 |
	   (__ilog2(MPC85XX_PCI1_UPPER_MEM - MPC85XX_PCI1_LOWER_MEM + 1) - 1);

	/* Setup outbound IO windows @ MPC85XX_PCI1_IO_BASE */
	pci->potar2 = (MPC85XX_PCI1_LOWER_IO >> 12) & 0x000fffff;
	pci->potear2 = 0x00000000;
	pci->powbar2 = (MPC85XX_PCI1_IO_BASE >> 12) & 0x000fffff;
	/* Enable, IO R/W */
	pci->powar2 = 0x80088000 | (__ilog2(MPC85XX_PCI1_IO_SIZE) - 1);

	/* Setup 2G inbound Memory Window @ 0 */
	pci->pitar1 = 0x00000000;
	pci->piwbar1 = 0x00000000;
	pci->piwar1 = 0xa0f5501e;	/* Enable, Prefetch, Local */
}


extern int mpc85xx_map_irq(struct pci_dev *dev, unsigned char idsel, unsigned char pin);
extern int mpc85xx_exclude_device(u_char bus, u_char devfn);

#ifdef CONFIG_85xx_PCI2
static void __init
mpc85xx_setup_pci2(struct pci_controller *hose)
{
	volatile struct ccsr_pci *pci;
	unsigned short temps;
	bd_t *binfo = (bd_t *) __res;

	pci = ioremap(binfo->bi_immr_base + MPC85xx_PCI2_OFFSET,
		    MPC85xx_PCI2_SIZE);

	early_read_config_word(hose, hose->bus_offset, 0, PCI_COMMAND, &temps);
	temps |= PCI_COMMAND_SERR | PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY;
	early_write_config_word(hose, hose->bus_offset, 0, PCI_COMMAND, temps);
	early_write_config_byte(hose, hose->bus_offset, 0, PCI_LATENCY_TIMER, 0x80);

	/* Disable all windows (except powar0 since its ignored) */
	pci->powar1 = 0;
	pci->powar2 = 0;
	pci->powar3 = 0;
	pci->powar4 = 0;
	pci->piwar1 = 0;
	pci->piwar2 = 0;
	pci->piwar3 = 0;

	/* Setup Phys:PCI 1:1 outbound mem window @ MPC85XX_PCI2_LOWER_MEM */
	pci->potar1 = (MPC85XX_PCI2_LOWER_MEM >> 12) & 0x000fffff;
	pci->potear1 = 0x00000000;
	pci->powbar1 = (MPC85XX_PCI2_LOWER_MEM >> 12) & 0x000fffff;
	/* Enable, Mem R/W */
	pci->powar1 = 0x80044000 |
	   (__ilog2(MPC85XX_PCI2_UPPER_MEM - MPC85XX_PCI2_LOWER_MEM + 1) - 1);

	/* Setup outbound IO windows @ MPC85XX_PCI2_IO_BASE */
	pci->potar2 = (MPC85XX_PCI2_LOWER_IO >> 12) & 0x000fffff;;
	pci->potear2 = 0x00000000;
	pci->powbar2 = (MPC85XX_PCI2_IO_BASE >> 12) & 0x000fffff;
	/* Enable, IO R/W */
	pci->powar2 = 0x80088000 | (__ilog2(MPC85XX_PCI2_IO_SIZE) - 1);

	/* Setup 2G inbound Memory Window @ 0 */
	pci->pitar1 = 0x00000000;
	pci->piwbar1 = 0x00000000;
	pci->piwar1 = 0xa0f5501e;	/* Enable, Prefetch, Local
					   Mem, Snoop R/W, 2G */
}
#endif /* CONFIG_85xx_PCI2 */

void __init
mpc85xx_setup_hose(void)
{
	int i;
	struct pci_controller *hose_a;
#ifdef CONFIG_85xx_PCI2
	struct pci_controller *hose_b;
#endif
	bd_t *binfo = (bd_t *) __res;

	printk(KERN_CRIT "mpc85xx_setup: Doing Pcie bridge setup\n");
	if (silkworm.board == SWBD68) { 
		struct pci_controller *hose_c;  /* For PCI Express */
	
		printk(KERN_CRIT "Scanning PcieBus...\n");
    		hose_c = pcibios_alloc_controller();

		if (!hose_c)
			return;

		ppc_md.pci_swizzle = common_swizzle;
		ppc_md.pci_map_irq = mpc85xx_map_irq;

		hose_c->first_busno = 0;
		hose_c->bus_offset = 0;
		hose_c->last_busno = 0xff;

		setup_indirect_pex(hose_c, 
			binfo->bi_immr_base + PEX_CFG_ADDR_OFFSET,
			binfo->bi_immr_base + PEX_CFG_DATA_OFFSET);

		mpc85xx_setup_pex(hose_c);

		hose_c->pci_mem_offset = MPC85XX_PEX_MEM_OFFSET;
		hose_c->mem_space.start = MPC85XX_PEX_LOWER_MEM;
		hose_c->mem_space.end = MPC85XX_PEX_UPPER_MEM;

		hose_c->io_space.start = MPC85XX_PEX_LOWER_IO;
		hose_c->io_space.end = MPC85XX_PEX_UPPER_IO;
		hose_c->io_base_phys = MPC85XX_PEX_IO_BASE;
		isa_io_base = (unsigned long) ioremap(MPC85XX_PEX_IO_BASE, 
				MPC85XX_PEX_IO_SIZE);
		hose_c->io_base_virt = (void *) isa_io_base;

  		/* setup resources */
		pci_init_resource(&hose_c->mem_resources[0], 
			MPC85XX_PEX_LOWER_MEM,
			MPC85XX_PEX_UPPER_MEM, IORESOURCE_MEM, 
			"PCI Express host bridge");

		pci_init_resource(&hose_c->io_resource, MPC85XX_PEX_LOWER_IO,
			MPC85XX_PEX_UPPER_IO, IORESOURCE_IO, 
			"PCI Express host bridge");

		ppc_md.pci_exclude_device = mpc85xx_exclude_device;
	}

	hose_a = pcibios_alloc_controller();

	if (!hose_a)
		return;

	ppc_md.pci_swizzle = common_swizzle;
	ppc_md.pci_map_irq = mpc85xx_map_irq;

	hose_a->first_busno = MPC85XX_PCI1_FIRST_BUSNO;
	hose_a->bus_offset = MPC85XX_PCI1_BUS_OFFSET;
	hose_a->last_busno = MPC85XX_PCI1_LAST_BUSNO;

	setup_indirect_pci(hose_a, binfo->bi_immr_base + PCI1_CFG_ADDR_OFFSET,
			   binfo->bi_immr_base + PCI1_CFG_DATA_OFFSET);
	hose_a->set_cfg_type = 1;

	mpc85xx_setup_pci1(hose_a);

	hose_a->pci_mem_offset = MPC85XX_PCI1_MEM_OFFSET;
	hose_a->mem_space.start = MPC85XX_PCI1_LOWER_MEM;
	hose_a->mem_space.end = MPC85XX_PCI1_UPPER_MEM;

	hose_a->io_space.start = MPC85XX_PCI1_LOWER_IO;
	hose_a->io_space.end = MPC85XX_PCI1_UPPER_IO;
	hose_a->io_base_phys = MPC85XX_PCI1_IO_BASE;
#ifdef CONFIG_85xx_PCI2
	hose_a->io_base_virt =  ioremap(MPC85XX_PCI1_IO_BASE,
					MPC85XX_PCI1_IO_SIZE +
					MPC85XX_PCI2_IO_SIZE);
#else
	hose_a->io_base_virt =  ioremap(MPC85XX_PCI1_IO_BASE,
					MPC85XX_PCI1_IO_SIZE);
#endif
	isa_io_base = (unsigned long)hose_a->io_base_virt;

	/* setup resources */
	pci_init_resource(&hose_a->mem_resources[0],
			MPC85XX_PCI1_LOWER_MEM,
			MPC85XX_PCI1_UPPER_MEM,
			IORESOURCE_MEM, "PCI1 host bridge");

	pci_init_resource(&hose_a->io_resource,
			MPC85XX_PCI1_LOWER_IO,
			MPC85XX_PCI1_UPPER_IO,
			IORESOURCE_IO, "PCI1 host bridge");

	ppc_md.pci_exclude_device = mpc85xx_exclude_device;

#if defined(CONFIG_MPC8555_CDS) || defined(CONFIG_MPC8548_CDS)
	/* Pre pciauto_bus_scan VIA init */
	mpc85xx_cds_enable_via(hose_a);
#endif

	/* The SILKWORM bootrom sets up the PCI BARs, skip it */
	if (silkworm_pci_standalone_scan) {
		printk("SILKWORM: PCI Standalone scan enabled on this platform\n");
		hose_a->last_busno = pciauto_bus_scan(hose_a, hose_a->first_busno);
	}

#if defined(CONFIG_MPC8555_CDS) || defined(CONFIG_MPC8548_CDS)
	/* Post pciauto_bus_scan VIA fixup */
	mpc85xx_cds_fixup_via(hose_a);
#endif

#ifdef CONFIG_85xx_PCI2
	hose_b = pcibios_alloc_controller();

	if (!hose_b)
		return;

	hose_b->bus_offset = hose_a->last_busno + 1;
	hose_b->first_busno = hose_a->last_busno + 1;
	hose_b->first_busno = hose_a->last_busno + 1;
	hose_b->last_busno = 0xff;

	setup_indirect_pci(hose_b, binfo->bi_immr_base + PCI2_CFG_ADDR_OFFSET,
			   binfo->bi_immr_base + PCI2_CFG_DATA_OFFSET);
	hose_b->set_cfg_type = 1;

	mpc85xx_setup_pci2(hose_b);

	hose_b->pci_mem_offset = MPC85XX_PCI2_MEM_OFFSET;
	hose_b->mem_space.start = MPC85XX_PCI2_LOWER_MEM;
	hose_b->mem_space.end = MPC85XX_PCI2_UPPER_MEM;

	hose_b->io_space.start = MPC85XX_PCI2_LOWER_IO;
	hose_b->io_space.end = MPC85XX_PCI2_UPPER_IO;
	hose_b->io_base_phys = MPC85XX_PCI2_IO_BASE;
	hose_b->io_base_virt = hose_a->io_base_virt + MPC85XX_PCI1_IO_SIZE;
	
	/* setup resources */
	pci_init_resource(&hose_b->mem_resources[0],
			MPC85XX_PCI2_LOWER_MEM,
			MPC85XX_PCI2_UPPER_MEM,
			IORESOURCE_MEM, "PCI2 host bridge");

	pci_init_resource(&hose_b->io_resource,
			MPC85XX_PCI2_LOWER_IO,
			MPC85XX_PCI2_UPPER_IO,
			IORESOURCE_IO, "PCI2 host bridge");

	hose_b->last_busno = pciauto_bus_scan(hose_b, hose_b->first_busno);

	/* let board code know what the last bus number was on PCI1 */
	mpc85xx_pci1_last_busno = hose_a->last_busno;
#endif /* PCI2 */

	switch (silkworm.board) {

		case SWBD76: /* ELARA */
		case SWBD133: /* Odin */
			{
				extern int pci_assign_all_busses;
				pci_assign_all_busses = 1;
			}
				/* Fall thru */
#if 0 /* @kevin -- for the without Co-CPU */
		case SWBD62: /* Zentron in Neptune */
		case SWBD77: /* Zentron in Pluto */
#endif
		//case SWBD68: /* Do NOT scan pci-e on Lance (Vader not Ready) */
		{
		struct pci_controller *hose_c;  /* For PCI Express */
	
    		hose_c = pcibios_alloc_controller();

		if (!hose_c)
			return;

		ppc_md.pci_swizzle = common_swizzle;
		ppc_md.pci_map_irq = mpc85xx_map_irq;

		if (silkworm_pci_standalone_scan) {
			hose_c->first_busno = hose_a->last_busno + 1;
			hose_c->bus_offset = hose_a->last_busno + 1;
			hose_c->last_busno = 0xff;
		} else {
			hose_c->first_busno = MPC85XX_PEX1_FIRST_BUSNO;
			hose_c->bus_offset = MPC85XX_PEX1_BUS_OFFSET;
			hose_c->last_busno = MPC85XX_PEX1_LAST_BUSNO;

		}

		setup_indirect_pex(hose_c, 
			binfo->bi_immr_base + PEX_CFG_ADDR_OFFSET,
			binfo->bi_immr_base + PEX_CFG_DATA_OFFSET);

		mpc85xx_setup_pex(hose_c);

		hose_c->set_cfg_type = 1;

		hose_c->pci_mem_offset = MPC85XX_PEX_MEM_OFFSET;
		hose_c->mem_space.start = MPC85XX_PEX_LOWER_MEM;
		hose_c->mem_space.end = MPC85XX_PEX_UPPER_MEM;

		hose_c->io_space.start = MPC85XX_PEX_LOWER_IO;
		hose_c->io_space.end = MPC85XX_PEX_UPPER_IO;
		hose_c->io_base_phys = MPC85XX_PEX_IO_BASE;
		isa_io_base = (unsigned long) ioremap(MPC85XX_PEX_IO_BASE, 
				MPC85XX_PEX_IO_SIZE);
		hose_c->io_base_virt = (void *) isa_io_base;

  		/* setup resources */
		pci_init_resource(&hose_c->mem_resources[0], 
			MPC85XX_PEX_LOWER_MEM,
			MPC85XX_PEX_UPPER_MEM, IORESOURCE_MEM, 
			"PCI Express host bridge");

		pci_init_resource(&hose_c->io_resource, MPC85XX_PEX_LOWER_IO,
			MPC85XX_PEX_UPPER_IO, IORESOURCE_IO, 
			"PCI Express host bridge");

		ppc_md.pci_exclude_device = mpc85xx_exclude_device;
		/* The SILKWORM bootrom sets up the PCI BARs, skip it */
		if (silkworm_pci_standalone_scan) {
			printk("SILKWORM: PCI-express standalone scan enabled on this platform\n");
			hose_c->last_busno = pciauto_bus_scan(hose_c, hose_c->first_busno);
			/* XXX HACK Alert: This is to force a scan across all buses, will need to revisit this soon */
			for (i=hose_c->first_busno; i< 4; ++i)
				hose_c->last_busno = pciauto_bus_scan(hose_c, i);

			mpc85xx_pci1_last_busno = hose_c->last_busno;
		}
		break;
		}
	default:
		break;
	}
	return;
}


#endif /* CONFIG_PCI */
