/*
 *
 *    Copyright (c) 1996-2006 Brocade Communications Systems, Inc.
 *    All rights reserved.
 *
 *    Description:
 *      Routines for xfs notification handling including registering,
 * 	unregistering and sending.
 *	Notifications are for xfs related events such force_shutdown.
 *
 */

#include <linux/module.h>
#include <linux/stddef.h>
#include <linux/xfs_notifier.h>

#if defined(CONFIG_SILKWORM)
/* Nothing, just so that I can trace which is Brocade specific file */
#endif

/* Static Variables */

/*
 * A notifier chain that calls those registered on xfs events including
 * xfs_force_shutdown.
 */
static struct notifier_block *xfs_notifierchain = NULL;


/*
 * int register_xfs_notifier(struct notifier_block *nb)
 *
 * Description:
 *   Called to add a notifier to the xfs notifier chain.
 *
 * Input(s):
 *   nb - The notifier to be added.
 *
 * Output(s):
 *   N/A
 *
 * Returns:
 *   N/A
 *
 */
int
register_xfs_notifier(struct notifier_block *nb)
{
	return notifier_chain_register(&xfs_notifierchain, nb);
}

/*
 * int unregister_xfs_notifier(struct notifier_block *nb)
 *
 * Description:
 *   Called to removed a earlier added notifier from the xfs notifier chain.
 *
 * Input(s):
 *   nb - The notifier to be removed.
 *
 * Output(s):
 *   N/A
 *
 * Returns:
 *   N/A
 *
 */
int
unregister_xfs_notifier(struct notifier_block *nb)
{
	return notifier_chain_unregister(&xfs_notifierchain, nb);
}

/*
 * int call_xfs_notifier(unsigned long notificationtype, void *typeinformation)
 *
 * Description:
 *   Called to send the notification and some information to all those on the
 * xfs notifier chain.
 *
 * Input(s):
 *   notificationtype - The notification to be sent.
 *   typeinformation - Information about the notification, type and
 *   contents vary with each notification.
 *
 * Output(s):
 *   N/A
 *
 * Returns:
 *   N/A
 *
 */
int
call_xfs_notifier(unsigned long notificationtype, void *typeinformation)
{
	return notifier_call_chain(&xfs_notifierchain, notificationtype,
		typeinformation);
}

/* Export list so that kernel modules can call the methods */
EXPORT_SYMBOL(register_xfs_notifier);
EXPORT_SYMBOL(unregister_xfs_notifier);
