#ifndef __FLASH_DUMPER_H
#define __FLASH_DUMPER_H

#ifdef __KERNEL__
/* These functions should only be called from a panic() handler,
 * or other function where interrupts are disabled and the system
 * is about to be rebooted.  Otherwise race conditions will occur with
 * the MTD device driver.
 */

/* flash_dump_open() - Only one open instance of dump device at a time.
 * This simple open will just rewind to the start of the flash partition.
 */
int flash_dump_open(void);

/* flash_dump_close() - This simple close will just close the device.
 */
int flash_dump_close(void);

/*
 * flash_dump_write() - will write 'size' bytes to the flash, erasing flash
 * and kicking the watchdog as necessary.  Our flash memory only handles
 * 32-bit aligned writes.  The flash data pointer and the input buffer
 * may NOT be 32-bit aligned.  The code handles it either way.
 */
int flash_dump_write(char *buf, unsigned long size);

/* dumper_init() - call this early in the boot process to
 * initialize the dumper code.
 */
int flash_dump_init(void *base, unsigned long size, int width, int sector_size,
	char *mtdname, uint8_t *cpld_addr, uint8_t vpp_offset, uint8_t vpp_mask);

#endif /* __KERNEL__ */
#endif /* __FLASH_DUMPER_H */
